<?php

class WPZOOM_List_Table_Checkbox_Option_Type {

	public $type = 'checkbox';
	public $context = 'list_table';
	public $option;

	public function __construct( $option ) {

		$this->option = $option;
		$this->option = wp_parse_args( $option, $this->get_defaults() );
		add_action( 'admin_enqueue_scripts', array( $this, 'admin_enqueue' ) );
	}

	public function admin_enqueue() {
		wp_enqueue_style(
			$this->get_action_name(),
			get_template_directory_uri() . '/functions/list-table-checkbox/assets/css/style.css',
			array(),
			WPZOOM::$themeVersion
		);
		wp_enqueue_script(
			$this->get_action_name(),
			get_template_directory_uri() . '/functions/list-table-checkbox/assets/js/list-table-checkbox.js',
			array(
				'jquery',
				'wp-util'
			),
			WPZOOM::$themeVersion,
			true
		);
	}

	public function get_ajax_action_name() {
		return sprintf( 'wp_ajax_%s', $this->get_action_name() );
	}

	public function get_action_name() {
		return sprintf( '%s_%s', $this->type, $this->context );
	}

	public function render( $post_id ) {

		$out = sprintf( '<div id="%s" class="list-table-checkbox-wrapper">', $this->option['id'] );
		$out .= '<p>';
		$out .= sprintf( '<input type="checkbox" name="%s" value="1" %s  class="list-table-checkbox"/>', $this->option['name'], checked( $this->get( $post_id ), true, false ) );
		$out .= sprintf( '<input type="hidden" name="%s" value="%s" />', 'post_id', $post_id );
		$out .= '<span class="dashicons dashicons-star-' . ( $this->get( $post_id ) ? 'filled' : 'empty' ) . '"></span>';
		$out .= wp_nonce_field( $this->get_ajax_action_name(), $this->get_action_name(), true, false );
		$out .= '</p>';
		$out .= '</div>';

		return $out;
	}

	public function ajax_callback() {

		if ( ! empty( $_POST['post_id'] ) &&
		     ! empty( $_POST[ $this->get_action_name() ] ) &&
		     wp_verify_nonce( $_POST[ $this->get_action_name() ], $this->get_ajax_action_name() )
		) {
			$post_id = $_POST['post_id'];
			$value   = $_POST[ $this->option['name'] ];
			update_metadata( 'post', $post_id, $this->option['name'], $value );

			wp_send_json_success( array(
				$post_id,
				$this->option['name'],
				$value
			) );
		}

		wp_send_json_error( array( 'message' => 'Invalid nonce' ) );
	}


	public function get( $post_id ) {
		$is_meta = metadata_exists( 'post', $post_id, $this->option['name'] );

		return $is_meta ?
			wp_validate_boolean( get_metadata( 'post', $post_id, $this->option['name'], true ) ) :
			$this->option['value'];
	}

	public function get_column_header( $columns ) {
		$columns[ $this->option['id'] ] = $this->option['title'];

		return $columns;
	}

	public function get_column_content( $column, $post_id ) {

		if ( $column === $this->option['id'] ) {
			echo $this->render( $post_id );
		}
	}

	public function get_defaults() {
		return array(
			'title'     => 'Checkbox',
			'desc'      => 'Description',
			'value'     => true,
			'type'      => 'checkbox',
			'name'      => $this->option['id'],
			'context'   => $this->context,
			'show'      => true,
			'post_type' => 'post'
		);
	}
}